var alert = true;
var delay = null;
var status = null;
var oldLocation = null;
var curLocation = null;
const keyboard = require("scripts/keyboard");

const isX = $device.isIphoneX;
const today = $app.env !== $env.app ? true : false;
const offset = today ? 0 : $device.isIphoneX ? 40 : 20;

const oneLayout = (make, view) => {
  status = null;
  let vsh = view.super.height;
  make.top.inset(offset);
  make.left.right.inset(0);
  make.height
    .equalTo(vsh)
    .dividedBy(2)
    .offset(-offset / 2);
};
const twoLayout = (make, view) => {
  make.left.right.bottom.inset(0);
  make.top.equalTo(view.prev.bottom);
};
const delLayout = (make, view) => {
  let os = today ? -5 : -10;
  make.bottom.inset(-1);
  make.size.equalTo($size(26, 32));
  make.right.equalTo(view.prev.left).offset(os);
};
const copyLayout = (make, view) => {
  make.bottom.inset(5);
  make.right.inset(today ? 5 : isX ? 20 : 5);
};
const tipsLayout = (make, view) => {
  let s = today ? 40 : 60;
  let o = today ? 0 : 60;
  make.left.inset(40);
  addShadows(view, s / 2);
  make.size.equalTo($size(s, s));
  make.centerY
    .equalTo(view.super)
    .dividedBy(2)
    .offset(-o);
};
const addShadows = (view, radius) => {
  //在layout中使用即可 给Views添加阴影
  var layer = view.runtimeValue().invoke("layer");
  layer.invoke("setShadowRadius", 5);
  layer.invoke("setShadowOpacity", 0.7);
  layer.invoke("setShadowOffset", $size(3, 3));
  layer.invoke("setCornerRadius", radius ? radius : 10);
  layer.invoke(
    "setShadowColor",
    $color("gray")
      .runtimeValue()
      .invoke("CGColor")
  );
};
const topTextScroll = sender => {
  let layer = $("bottomText");
  let offset = sender.contentOffset;
  if (!layer.dragging && !layer.tracking)
    layer.scrollToOffset($point(0, offset.y <= 0 ? 0 : offset.y));
};
const btmTextScroll = sender => {
  let layer = $("topText");
  let offset = sender.contentOffset;
  if (!layer.dragging && !layer.tracking)
    layer.scrollToOffset($point(0, offset.y <= 0 ? 0 : offset.y));
};

const topView = {
  type: "view",
  props: {
    id: "topView"
  },
  layout: oneLayout,
  views: [
    {
      type: "text",
      props: {
        id: "topText",
        bgcolor: $color("white"),
        insets: $insets(14, 10, 12, 10),
        textColor: $color("lightGray"),
        editable: today ? false : true
      },
      layout: $layout.fill,
      events: {
        didScroll: topTextScroll,
        didEndDragging: topTextScroll,
        didEndDecelerating: topTextScroll,
        willBeginDecelerating: topTextScroll
      }
    },
    {
      type: "view",
      props: {
        id: "topRuler",
        bgcolor: $color("#4484f4")
      },
      layout: (make, view) => {
        make.left.inset(10);
        make.width.equalTo(5);
        make.height.equalTo(0);
        make.bottom.equalTo(view.super);
      },
      views: [
        {
          type: "view",
          views: [
            {
              type: "view",
              props: {
                alpha: 0,
                circular: 1,
                id: "topBall_100",
                bgcolor: $color("#4484f4")
              },
              layout: (make, view) => {
                make.size.equalTo($size(20, 20));
                make.bottom.equalTo(view.super);
                make.centerX.equalTo(view.super);
              }
            },
            {
              type: "view",
              props: {
                alpha: 0,
                circular: 1,
                id: "topBall_100_point",
                bgcolor: $color("#4484f4")
              },
              layout: (make, view) => {
                make.size.equalTo($size(8, 8));
                make.centerY.equalTo(view.prev);
                make.centerX.equalTo(view.super).offset(20);
              }
            },
            {
              type: "label",
              props: {
                alpha: 0,
                circular: 1,
                text: "向下展开",
                align: $align.center,
                id: "topBall_100_face",
                bgcolor: $color("#4484f4"),
                textColor: $color("white")
              },
              layout: (make, view) => {
                make.width.equalTo(40);
                make.height.equalTo(40);
                make.centerY.equalTo(view.prev);
                make.centerX.equalTo(view.super).offset(50);
              }
            }
          ],
          layout: (make, view) => {
            make.centerX.equalTo(view.super);
            make.bottom.equalTo(view.super).offset(-100);
          }
        },
        {
          type: "view",
          views: [
            {
              type: "view",
              props: {
                alpha: 0,
                circular: 1,
                id: "topBall_150",
                bgcolor: $color("#4484f4")
              },
              layout: (make, view) => {
                make.size.equalTo($size(20, 20));
                make.bottom.equalTo(view.super);
                make.centerX.equalTo(view.super);
              }
            },
            {
              type: "view",
              props: {
                alpha: 0,
                circular: 1,
                id: "topBall_150_point",
                bgcolor: $color("#4484f4")
              },
              layout: (make, view) => {
                make.size.equalTo($size(8, 8));
                make.centerY.equalTo(view.prev);
                make.centerX.equalTo(view.super).offset(20);
              }
            },
            {
              type: "label",
              props: {
                alpha: 0,
                circular: 1,
                text: "向上展开",
                align: $align.center,
                id: "topBall_150_face",
                bgcolor: $color("#4484f4"),
                textColor: $color("white")
              },
              layout: (make, view) => {
                make.width.equalTo(40);
                make.height.equalTo(40);
                make.centerY.equalTo(view.prev);
                make.centerX.equalTo(view.super).offset(50);
              }
            }
          ],
          layout: (make, view) => {
            make.centerX.equalTo(view.super);
            make.bottom.equalTo(view.super).offset(-150);
          }
        }
      ]
    },
    {
      type: "button",
      props: {
        alpha: 0.7,
        bgcolor: $color("clear"),
        icon: $icon("021", $color("lightGray"), $size(20, 20))
      },
      layout: copyLayout,
      events: {
        tapped(sender) {
          $device.taptic(1);
          $("topText").text =
            $app.env !== $env.keyboard ? "" : ($clipboard.text = "");
        }
      }
    },
    {
      type: "button",
      props: {
        alpha: 0,
        id: "delBtn",
        src: "assets/del.png",
        bgcolor: $color("clear")
      },
      layout: delLayout,
      events: {
        tapped(sender) {
          $device.taptic(1);
          $keyboard.delete();
        }
      }
    }
  ]
};
const tipsView = {
  type: "view",
  props: {
    alpha: 0,
    id: "tipsView",
    bgcolor: $color("white")
  },
  views: [
    {
      type: "image",
      props: {
        id: "tipsImg"
      },
      layout: $layout.fill
    }
  ],
  layout: tipsLayout
};
const blurView = {
  type: "blur",
  props: {
    alpha: 0,
    style: 1, // 0 ~ 5
    userInteractionEnabled: 1
  },
  layout: $layout.fill
};
const bottomView = {
  type: "view",
  props: {
    id: "bottomView"
  },
  layout: twoLayout,
  views: [
    {
      type: "text",
      props: {
        id: "bottomText",
        editable: false,
        bgcolor: $color("#4484f4"),
        textColor: $color("white"),
        insets: $insets(14, 10, 12, 10)
      },
      layout: $layout.fill,
      events: {
        didScroll: btmTextScroll,
        didEndDragging: btmTextScroll,
        didEndDecelerating: btmTextScroll,
        willBeginDecelerating: btmTextScroll
      }
    },
    {
      type: "view",
      props: {
        id: "botRuler",
        bgcolor: $color("white")
      },
      layout: (make, view) => {
        make.left.inset(10);
        make.width.equalTo(5);
        make.height.equalTo(0);
        make.top.equalTo(view.super).offset(-2.5);
      },
      views: [
        {
          type: "view",
          views: [
            {
              type: "view",
              props: {
                alpha: 0,
                circular: 1,
                id: "botBall_100",
                bgcolor: $color("white")
              },
              layout: (make, view) => {
                make.size.equalTo($size(20, 20));
                make.top.equalTo(view.super);
                make.centerX.equalTo(view.super);
              }
            },
            {
              type: "view",
              props: {
                alpha: 0,
                circular: 1,
                id: "botBall_100_point",
                bgcolor: $color("white")
              },
              layout: (make, view) => {
                make.size.equalTo($size(8, 8));
                make.centerY.equalTo(view.prev);
                make.centerX.equalTo(view.super).offset(20);
              }
            },
            {
              type: "label",
              props: {
                alpha: 0,
                circular: 1,
                text: "向下展开",
                align: $align.center,
                id: "botBall_100_face",
                bgcolor: $color("white"),
                textColor: $color("lightGray")
              },
              layout: (make, view) => {
                make.width.equalTo(40);
                make.height.equalTo(40);
                make.centerY.equalTo(view.prev);
                make.centerX.equalTo(view.super).offset(50);
              }
            }
          ],
          layout: (make, view) => {
            make.centerX.equalTo(view.super);
            make.top.equalTo(view.super).offset(100);
          }
        },
        {
          type: "view",
          views: [
            {
              type: "view",
              props: {
                alpha: 0,
                circular: 1,
                id: "botBall_150",
                bgcolor: $color("white")
              },
              layout: (make, view) => {
                make.size.equalTo($size(20, 20));
                make.top.equalTo(view.super);
                make.centerX.equalTo(view.super);
              }
            },
            {
              type: "view",
              props: {
                alpha: 0,
                circular: 1,
                id: "botBall_150_point",
                bgcolor: $color("white")
              },
              layout: (make, view) => {
                make.size.equalTo($size(8, 8));
                make.centerY.equalTo(view.prev);
                make.centerX.equalTo(view.super).offset(20);
              }
            },
            {
              type: "label",
              props: {
                alpha: 0,
                circular: 1,
                text: "向下展开",
                align: $align.center,
                id: "botBall_150_face",
                bgcolor: $color("white"),
                textColor: $color("lightGray")
              },
              layout: (make, view) => {
                make.width.equalTo(40);
                make.height.equalTo(40);
                make.centerY.equalTo(view.prev);
                make.centerX.equalTo(view.super).offset(50);
              }
            }
          ],
          layout: (make, view) => {
            make.centerX.equalTo(view.super);
            make.top.equalTo(view.super).offset(150);
          }
        }
      ]
    },
    {
      type: "button",
      props: {
        alpha: 0.7,
        bgcolor: $color("clear"),
        icon: $icon("019", $color("lightGray"), $size(20, 20))
      },
      layout: copyLayout,
      events: {
        tapped(sender) {
          $device.taptic(1);
          $app.env !== $env.keyboard
            ? ($clipboard.text = $("bottomText").text)
            : $keyboard.insert($("bottomText").text);
        }
      }
    }
  ]
};
const buttonView = {
  type: "button",
  props: {
    circular: 1,
    id: "googleButton",
    bgcolor: $color("white"),
    src: "assets/google.png"
  },
  layout: (make, view) => {
    let top = $("topView");
    make.size.equalTo($size(30, 30));
    make.centerX.equalTo(view.super);
    make.centerY.equalTo(top.bottom); //.offset(today ? 0 : offset / 2)
  },
  events: {
    tapped: async sender => {
      await googleTrans(
        $("topText").text.length ? $("topText").text : $clipboard.text
      );
    },
    longPressed: () => {
      $app.env == $env.today && $app.widgetIndex < 0 || $app.widgetIndex > 2 ? $app.close() : 0;
    },
    touchesBegan: (sender, location) => {
      $device.taptic(1);
      oldLocation = location;
    },
    touchesMoved: (sender, location) => {
      curLocation = location;
      let X_distance = Math.abs(curLocation.x - oldLocation.x);
      let Y_distance = Math.abs(curLocation.y - oldLocation.y);
      let X_direction = curLocation.x > oldLocation.x ? true : false; //ture为右
      let Y_direction = curLocation.y > oldLocation.y ? true : false; //ture为上
      //左右
      if (X_distance <= 150 && X_distance >= 100 && Y_distance <= 30) {
        if (X_direction) tipsViewAnimate("share");
      } else if (X_distance > 150 && Y_distance <= 30) {
        if (X_direction) tipsViewAnimate("exit");
      }
      //上下
      if (Y_distance < 100 && X_distance >= 20)
        botRulesViewAnimate(0) && topRulesViewAnimate(0);
      else {
        if (Y_direction) botRulesViewAnimate(Y_distance);
        else topRulesViewAnimate(Y_distance);
      }

      if (Y_distance <= 150 && Y_distance >= 100 && X_distance <= 20) {
        if (Y_direction && status == null) tipsViewAnimate("bot");
        else if (!Y_direction && status == null) tipsViewAnimate("top");
        else tipsViewAnimate("nol");
      } else if (Y_distance > 150) {
        if (Y_direction && status == false) tipsViewAnimate("bot");
        else if (!Y_direction && status == true) tipsViewAnimate("top");
      }
    },
    touchesEnded: (sender, location, locations) => {
      let X_distance = Math.abs(curLocation.x - oldLocation.x);
      let Y_distance = Math.abs(curLocation.y - oldLocation.y);
      let X_direction = curLocation.x > oldLocation.x ? true : false;
      let Y_direction = curLocation.y > oldLocation.y ? true : false;
      //左右
      if (X_distance > 150 && Y_distance <= 30) {
        if ($app.env == $env.keyboard) $keyboard.next();
        else if (!today) $app.close();
      } else if (X_distance <= 150 && X_distance >= 100 && Y_distance <= 30) {
        if (X_direction) snapshot();
      }
      //上下
      if (Y_distance <= 150 && Y_distance >= 100 && X_distance <= 20) {
        if (Y_direction) halfTop();
        else halfBottom();
      } else if (Y_distance > 150) {
        if (Y_direction) halfTop(1);
        else halfBottom(1);
      }
      botRulesViewAnimate(0) && topRulesViewAnimate(0);
    }
  }
};

function init() {
  return async () => {
    $ui.render({
      props: {
        id: "mainView",
        statusBarStyle: 0,
        navBarHidden: true,
        bgcolor: $color("white")
      },
      views: [topView, bottomView, tipsView, blurView, buttonView],
      events: {
        appeared: async () => {
          timer();
          keyboard.hideBar("mainView");
        }
      }
    });
    $delay(0.1, async () => {
      $clipboard.text ? googleTrans($clipboard.text) : 0;
    });
  };
}
function snapshot() {
  alert = false;
  $ui.alert({
    title: "是否分享该页面",
    actions: [
      {
        title: "否",
        handler: () => {
          alert = true;
        }
      },
      {
        title: "是",
        handler: () => {
          $delay(today ? 0 : 0.3, () => {
            $share.sheet([$("mainView").snapshot]);
          });
          alert = true;
        }
      }
    ]
  });
}
function whichLan(text) {
  let englishChar = text.match(/[a-zA-Z]/g);
  let englishNumber = !englishChar ? 0 : englishChar.length;
  let chineseChar = text.match(/[\u4e00-\u9fff\uf900-\ufaff]/g);
  let chineseNumber = !chineseChar ? 0 : chineseChar.length;

  return chineseNumber * 2 >= englishNumber && chineseNumber
    ? "zh-CN"
    : englishNumber >= 8
    ? "en"
    : "auto";
}
function animation(s, c, a) {
  $("googleButton").updateLayout(make => {
    make.size.equalTo($size(s, s));
  });
  $("googleButton").bgcolor = $color(c);
  $ui.animate({
    duration: 1,
    damping: 0.9,
    velocity: 0.6,
    animation: () => {
      $("blur").alpha = a;
      $("googleButton").relayout();
    }
  });
}
function halfTop(flag) {
  if (status == null || flag)
    changeViewAnimate((make, view) => {
      status = true;
      let value = today ? 40 : 80;
      let vsh = view.super.height;
      make.top.inset(offset);
      make.left.right.inset(0);
      make.height.equalTo(vsh).offset(-offset / 2 - value);
    });
  else if (status == false) changeViewAnimate(oneLayout);
}
function halfBottom(flag) {
  if (status == null || flag)
    changeViewAnimate(make => {
      status = false;
      make.top.inset(offset);
      make.height.equalTo(40);
      make.left.right.inset(0);
    });
  else if (status == true) changeViewAnimate(oneLayout);
}
function tipsViewAnimate($1) {
  $("tipsImg").src = `assets/${$1}.png`;
  const animate = (dura, func) => {
    $ui.animate({
      duration: dura,
      damping: 0.9,
      velocity: 0.6,
      animation: func
    });
  };
  animate(1, () => ($("tipsView").alpha = 1));
  if (delay) delay.cancel();
  delay = $delay(0.5, () => animate(1, () => ($("tipsView").alpha = 0)));
}
function topRulesViewAnimate($1) {
  $("topRuler").updateLayout((make, view) => {
    make.height.equalTo($1);
  });
  $("topBall_100_face").updateLayout((make, view) => {
    make.width.equalTo($1 >= 100 ? 80 : 40);
    make.centerX.equalTo(view.super).offset($1 >= 100 && $1 < 150 ? 70 : 50);
  });
  $("topBall_150_face").updateLayout((make, view) => {
    make.width.equalTo($1 >= 150 ? 80 : 40);
    make.centerX.equalTo(view.super).offset($1 >= 150 ? 70 : 50);
  });
  const animate = (a, c) => {
    $ui.animate({
      duration: 1,
      damping: 0.9,
      velocity: 0.6,
      animation: a,
      completion: c ? c : () => {}
    });
  };
  animate(() => {
    $("topBall_100_face").text = status == true ? "还原" : "向上展开";
    if ($1 <= 150 && $1 >= 100) {
      $("topBall_100").alpha = 1;
      $("topBall_100_face").alpha = 1;
      $("topBall_100_point").alpha = 1;
      $("topBall_150").alpha = 0;
      $("topBall_150_face").alpha = 0;
      $("topBall_150_point").alpha = 0;
    } else if ($1 > 150 && status != null) {
      $("topBall_100").alpha = 1;
      $("topBall_100_face").alpha = 0;
      $("topBall_100_point").alpha = 0;
      $("topBall_150").alpha = 1;
      $("topBall_150_face").alpha = 1;
      $("topBall_150_point").alpha = 1;
    } else if ($1 < 100) {
      $("topBall_100").alpha = 0;
      $("topBall_100_face").alpha = 0;
      $("topBall_100_point").alpha = 0;
      $("topBall_150").alpha = 0;
      $("topBall_150_face").alpha = 0;
      $("topBall_150_point").alpha = 0;
    }
    $("topRuler").relayout();
    animate(() => $("topBall_150_face").relayout());
    animate(() => $("topBall_100_face").relayout());
  });
  return true;
}
function botRulesViewAnimate($1) {
  $("botRuler").updateLayout((make, view) => {
    make.height.equalTo($1);
  });
  $("botBall_100_face").updateLayout((make, view) => {
    make.width.equalTo($1 >= 100 ? 80 : 40);
    make.centerX.equalTo(view.super).offset($1 >= 100 && $1 < 150 ? 70 : 50);
  });
  $("botBall_150_face").updateLayout((make, view) => {
    make.width.equalTo($1 >= 150 ? 80 : 40);
    make.centerX.equalTo(view.super).offset($1 >= 150 ? 70 : 50);
  });
  const animate = (a, c) => {
    $ui.animate({
      duration: 1,
      damping: 0.9,
      velocity: 0.6,
      animation: a,
      completion: c ? c : () => {}
    });
  };
  animate(() => {
    $("botBall_100_face").text = status == false ? "还原" : "向下展开";
    if ($1 <= 150 && $1 >= 100) {
      $("botBall_100").alpha = 1;
      $("botBall_100_face").alpha = 1;
      $("botBall_100_point").alpha = 1;
      $("botBall_150").alpha = 0;
      $("botBall_150_face").alpha = 0;
      $("botBall_150_point").alpha = 0;
    } else if ($1 > 150 && status != null) {
      $("botBall_100").alpha = 1;
      $("botBall_100_face").alpha = 0;
      $("botBall_100_point").alpha = 0;
      $("botBall_150").alpha = 1;
      $("botBall_150_face").alpha = 1;
      $("botBall_150_point").alpha = 1;
    } else if ($1 < 100) {
      $("botBall_100").alpha = 0;
      $("botBall_100_face").alpha = 0;
      $("botBall_100_point").alpha = 0;
      $("botBall_150").alpha = 0;
      $("botBall_150_face").alpha = 0;
      $("botBall_150_point").alpha = 0;
    }
    $("botRuler").relayout();
    animate(() => $("botBall_150_face").relayout());
    animate(() => $("botBall_100_face").relayout());
  });
  return true;
}
function changeViewAnimate(layout) {
  $("topView").remakeLayout(layout);
  $ui.animate({
    duration: 1,
    damping: 0.9,
    velocity: 0.6,
    animation: () => {
      $("topView").relayout();
    }
  });
}

async function timer() {
  $timer.schedule({
    interval: 1,
    handler: async () => {
      if ($app.env == $env.keyboard) {
        let sW = $clipboard.text;
        let iD = $("topText").text != sW;
        let hT = $keyboard.hasText || $keyboard.selectedText;
        if (sW && sW.length && iD) googleTrans(sW);
        if (hT) $("delBtn").alpha = 0.4;
        else $("delBtn").alpha = 0;

        console.log($("topText").dragging);
      }
    }
  });
}
async function request(m, u, h, b, t, s) {
  let d = await $http.request({
    method: m,
    url: u,
    timeout: t,
    header: h,
    body: b,
    showsProgress: s
  });
  return d.data;
}
async function tryTrans(sl, tl, text) {
  let or = (tr = "");
  let d = await request(
    "POST",
    "http://translate.google.cn/translate_a/single",
    {
      "User-Agent": "iOSTranslate",
      "Content-Type": "application/x-www-form-urlencoded"
    },
    { dt: "t", q: text, tl: tl, ie: "UTF-8", sl: sl, client: "ia", dj: "1" },
    5,
    false
  );
  d.sentences.forEach(e => {
    $("topText").text = or = or.concat(e.orig);
    $("bottomText").text = tr = tr.concat(e.trans);
  });
  $clipboard.text = $("topText").text;
}
async function googleTrans(text) {
  if (!text.trim().length) return;
  animation(64, "clear", 1);
  let sl = whichLan(text);
  let tl = sl == "zh-CN" ? "en" : "zh-CN";
  await tryTrans(sl, tl, text);
  $("topText").text == $("bottomText").text
    ? await tryTrans("auto", sl, text)
    : 0;
  animation(30, "white", 0);
}

module.exports = {
  init: init
};
