const bridge = require("../../bridge");
const moduleName = "calendar";

exports.fetch = args => {
  args = args || {};
  args.startDate = bridge.toNativeDate(args.startDate);
  args.endDate = bridge.toNativeDate(args.endDate);
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "fetch", args, response => {
      response.events = (response.events || []).map(item => {
        const event = item;
        event.modifiedDate = bridge.toNodeDate(item.modifiedDate);
        event.creationDate = bridge.toNodeDate(item.creationDate);
        event.startDate = bridge.toNodeDate(item.startDate);
        event.endDate = bridge.toNodeDate(item.endDate);
        return event;
      });
      resolve(response);
    });
  });
}

exports.create = args => {
  args = args || {};
  args.startDate = bridge.toNativeDate(args.startDate);
  args.endDate = bridge.toNativeDate(args.endDate);
  args.alarmDate = bridge.toNativeDate(args.alarmDate);

  if (args.alarmDates) {
    args.alarmDates = args.alarmDates.map(date => bridge.toNativeDate(date));
  }

  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "create", args, resolve);
  });
}

exports.save = args => {
  args = args || {};
  args.startDate = bridge.toNativeDate(args.startDate);
  args.endDate = bridge.toNativeDate(args.endDate);
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "save", args, resolve);
  });
}

exports.delete = args => {
  args = args || {};
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "delete", args, resolve);
  });
}