const bridge = require("../../bridge");
const moduleName = "contact";

exports.pick = args => {
  args = args || {};
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "pick", args, response => {
      const results = args.multi ? response.results : response;
      resolve(results);
    });
  });
}

exports.fetch = args => {
  args = args || {};
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "fetch", args, response => {
      resolve(response.results);
    });
  });
}

exports.create = args => {
  args = args || {};
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "create", args, resolve);
  });
}

exports.save = args => {
  args = args || {};
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "save", args, resolve);
  });
}

exports.delete = args => {
  args = args || [];
  if (!Array.isArray(args)) {
    args = [args];
  }
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "delete", {
      objects: args
    }, resolve);
  });
}

exports.fetchGroups = () => {
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "fetchGroups", response => {
      resolve(response.results);
    });
  });
}

exports.addGroup = name => {
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "addGroup", {
      name: String(name)
    }, resolve);
  });
}

exports.deleteGroup = args => {
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "deleteGroup", {
      group: args
    }, resolve);
  });
}

exports.updateGroup = args => {
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "updateGroup", {
      group: args
    }, resolve);
  });
}

exports.addToGroup = args => {
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "addToGroup", args, resolve);
  });
}

exports.removeFromGroup = args => {
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "removeFromGroup", args, resolve);
  });
}