const bridge = require("../../bridge");
const moduleName = "imagekit";

function handleImage(method, file, value=null) {
  return handleImageWithValues(method, file, {value});
}

function handleImageWithValues(method, file, values={}) {
  if (file) {
    values.image = bridge.toNativeFile(file);
  }
  const result = bridge.invokeNative(moduleName, method, values);
  return bridge.toNodeFile(result);
}

exports.info = file => {
  return bridge.invokeNative(moduleName, "info", {
    image: bridge.toNativeFile(file)
  });
}

exports.grayscale = file => {
  return handleImage("grayscale", file);
}

exports.invert = file => {
  return handleImage("invert", file);
}

exports.sepia = file => {
  return handleImage("sepia", file);
}

exports.adjustEnhance = file => {
  return handleImage("adjustEnhance", file);
}

exports.adjustRedEye = file => {
  return handleImage("adjustRedEye", file);
}

exports.adjustBrightness = (file, value) => {
  return handleImage("adjustBrightness", file, value);
}

exports.adjustContrast = (file, value) => {
  return handleImage("adjustContrast", file, value);
}

exports.adjustGamma = (file, value) => {
  return handleImage("adjustGamma", file, value);
}

exports.adjustOpacity = (file, value) => {
  return handleImage("adjustOpacity", file, value);
}

exports.blur = (file, value) => {
  return handleImage("blur", file, value);
}

exports.emboss = (file, value) => {
  return handleImage("emboss", file, value);
}

exports.sharpen = (file, value) => {
  return handleImage("sharpen", file, value);
}

exports.unsharpen = (file, value) => {
  return handleImage("unsharpen", file, value);
}

exports.detectEdge = (file, value) => {
  return handleImage("detectEdge", file, value);
}

exports.mask = (file, mask) => {
  return handleImageWithValues("mask", file, {
    mask: bridge.toNativeFile(mask)
  });
}

exports.reflect = (file, height, fromAlpha, toAlpha) => {
  return handleImageWithValues("reflect", file, {
    height,
    fromAlpha,
    toAlpha
  });
}

exports.cropTo = (file, size, mode=0) => {
  return handleImageWithValues("cropTo", file, {
    size,
    mode
  });
}

exports.scaleBy = (file, value) => {
  return handleImage("scaleBy", file, value);
}

exports.scaleTo = (file, size, mode=0) => {
  return handleImageWithValues("scaleTo", file, {
    size,
    mode
  });
}

exports.scaleFill = (file, value) => {
  return handleImage("scaleFill", file, value);
}

exports.scaleAspectFit = (file, value) => {
  return handleImage("scaleAspectFit", file, value);
}

exports.scaleAspectFill = (file, value) => {
  return handleImage("scaleAspectFill", file, value);
}

exports.rotate = (file, value) => {
  return handleImage("rotate", file, value);
}

exports.rotatePixels = (file, value) => {
  return handleImage("rotatePixels", file, value);
}

exports.flip = (file, mode=0) => {
  return handleImage("flip", file, mode);
}

exports.concatenate = (files, space=0, mode=0) => {
  return handleImageWithValues("concatenate", null, {
    images: files.map(file => bridge.toNativeFile(file)),
    space,
    mode
  });
}

exports.combine = (file, mask, mode=8) => {
  return handleImageWithValues("combine", file, {
    mask: bridge.toNativeFile(mask),
    mode
  });
}

exports.rounded = (file, value) => {
  return handleImage("rounded", file, value);
}

exports.circular = file => {
  return handleImage("circular", file);
}

exports.extractGIF = file => {
  const image = bridge.toNativeFile(file);
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "extractGIF", {image}, response => {
      response.images = (response.images || []).map(image => {
        return bridge.toNodeFile(image);
      });
      resolve(response);
    });
  });
}

function makeAsset(method, source, options={}) {
  const sourceToFile = source => {
    if (Array.isArray(source)) {
      return source.map(file => bridge.toNativeFile(file));
    } else {
      return bridge.toNativeFile(source);
    }
  }

  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, method, {
      source: sourceToFile(source),
      options
    }, file => {
      resolve(bridge.toNodeFile(file));
    });
  });
}

exports.makeGIF = (source, options={}) => {
  return makeAsset("makeGIF", source, options);
}

exports.makeVideo = (source, options={}) => {
  return makeAsset("makeVideo", source, options);
}