const bridge = require("../../bridge");
const moduleName = "photo";

exports.take = args => {
  return _showImagePicker("take", args);
}

exports.pick = args => {
  return _showImagePicker("pick", args);
}

exports.scan = () => {
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "scan", response => {
      const results = response.status ? response.results.map($0 => {
        return bridge.toNodeFile($0);
      }) : [];
      resolve(results);
    });
  });
}

exports.save = file => {
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "save", {
      file: bridge.toNativeFile(file)
    }, result => {
      resolve(result.success);
    });
  });
}

exports.fetch = args => {
  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "fetch", args, response => {
      resolve(response.results.map($0 => bridge.toNodeFile($0)));
    });
  });
}

exports.delete = args => {
  return Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, "delete", args, result => {
      resolve(result.success);
    });
  });
}

function _showImagePicker(method, args) {
  const toResult = $0 => {
    return {
      data: bridge.toNodeFile($0.data),
      metadata: $0.metadata
    };
  };

  return new Promise((resolve, reject) => {
    bridge.invokeNative(moduleName, method, args, response => {
      if (response.results) {
        resolve({
          status: response.status,
          results: response.results.map($0 => toResult($0))
        });
      } else {
        resolve(toResult(response));
      }
    });
  });
}