const util = require("util");
const bridge = require("../../bridge");
const moduleName = "ui";

exports.alert = args => {
  if (typeof args !== "object" || Array.isArray(args)) {
    args = {
      title: util.format(args)
    };
  }
  args.sheet = false;
  return _alert(args);
}

exports.action = args => {
  if (typeof args !== "object") {
    args = {
      title: util.format(args)
    };
  }
  args.sheet = true;
  return _alert(args);
}

exports.menu = args => {
  if (Array.isArray(args)) {
    args = {
      items: args
    }
  }

  args = args || {};
  const title = args.title;
  const items = args.items;
  
  if (items && items.length > 10) {
    return new Promise((resolve, reject) => {
      bridge.invokeNative(moduleName, "menu", args, resolve);
    });
  } else {
    return _alert({
      title: title,
      actions: items,
      sheet: true
    });
  }
}

exports.toast = (message, delay) => {
  _showToast("toast", message, delay);
}

exports.success = (message, delay) => {
  _showToast("success", message, delay);
}

exports.warning = (message, delay) => {
  _showToast("warning", message, delay);
}

exports.error = (message, delay) => {
  _showToast("error", message, delay);
}

exports.clearToast = () => {
  bridge.invokeNative(moduleName, "clearToast");
}

exports.showHUD = message => {
  bridge.invokeNative(moduleName, "showHUD", {
    message: util.format(message)
  });
}

exports.hideHUD = () => {
  bridge.invokeNative(moduleName, "hideHUD");
}

exports.showProgress = (progress, message) => {
  if (typeof progress === "number" && typeof message === "string") {
    bridge.invokeNative(moduleName, "showProgress", {
      progress: progress,
      message: String(message)
    });
  } else {
    throw "Please make sure parameters are valid when using ui.showProgress(...)";
  }
}

exports.hideProgress = () => {
  bridge.invokeNative(moduleName, "hideProgress");
}

function _alert(args) {
  if (typeof args === "string") {
    args = {"title": args};
  }

  const invokeNative = () => {
    bridge.invokeNative(moduleName, "alert", args);
  }

  if (args.actions) {
    return new Promise((resolve, reject) => {
      args.actions = args.actions.map((title, index) => {
        return {
          title: util.format(title),
          hash: bridge.createHandler(() => {
            resolve({
              title: util.format(title),
              index: index
            });
          })
        }
      });
      args["hash"] = bridge.createHandler(() => {
        resolve({});
      });
      invokeNative();
    });
  } else {
    return invokeNative();
  }
}

function _showToast(method, message, delay) {
  bridge.invokeNative(moduleName, method, {
    message: util.format(message),
    delay: delay || 0
  }); 
}