module.exports = {
  "!name": "jsbox",
  "$jsbox": {
    "!doc": "Provides JSBox related APIs.",
    "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/node-jsbox",
    "version": {
      "!type": "string",
      "!doc": "Indicates the current running JSBox version.",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/node-jsbox?id=jsboxversion"
    },
    "path": {
      "!doc": "Useful paths in the file system.",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/node-jsbox?id=jsboxpath",
      "home": {
        "!type": "string",
        "!doc": "Home folder of the app.",
        "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/fs/paths"
      },
      "scripts": {
        "!type": "string",
        "!doc": "The folder that stores all scripts.",
        "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/fs/paths"
      },
      "current": {
        "!type": "string",
        "!doc": "The folder that contains the current running script.",
        "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/fs/paths"
      },
      "shared": {
        "!type": "string",
        "!doc": "The folder that is shared over JSBox's scripts.",
        "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/fs/paths"
      },
      "icloud": {
        "!type": "string",
        "!doc": "The iCloud Drive folder of JSBox.",
        "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/fs/paths"
      }
    },
    "run": {
      "!type": "fn(args: ?)",
      "!doc": "Invoke JSBox with script or name.",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/node-jsbox"
    },
    "notify": {
      "!type": "fn(id: string, result: ?)",
      "!doc": "Notify JSBox listener with id and result object.",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/jsbox-node"
    },
    "listen": {
      "!type": "fn(id: string, callback: fn(result: ?))",
      "!doc": "Listen notifications from Node.js runtime.",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/jsbox-node"
    },
    "stdin": {
      "!type": "object",
      "!doc": "A replacement of process.stdin, based on JSBox's runtime.",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/native-modules/input"
    },
    "stdout": {
      "!type": "object",
      "!doc": "A replacement of process.stdout, based on JSBox's runtime.",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/native-modules/input"
    }
  },
  "$context": {
    "!doc": "Provides APIs to access data from extensions.",
    "!url": "",
    "query": {
      "!type": "object",
      "!doc": "Object that stores the passed in parameters.",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/context?id=contextquery"
    },
    "text": {
      "!type": "string",
      "!doc": "Get the first text from share sheet",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/context?id=contexttextimagelinksafari"
    },
    "textItems": {
      "!type": "[string]",
      "!doc": "Get all text items from share sheet",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/context?id=contexttextimagelinksafari"
    },
    "image": {
      "!type": "+Buffer",
      "!doc": "Get the first image from share sheet",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/context?id=contexttextimagelinksafari"
    },
    "imageItems": {
      "!type": "[+Buffer]",
      "!doc": "Get all image items from share sheet",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/context?id=contexttextimagelinksafari"
    },
    "link": {
      "!type": "string",
      "!doc": "Get the first link from share sheet",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/context?id=contexttextimagelinksafari"
    },
    "linkItems": {
      "!type": "[string]",
      "!doc": "Get all link items from share sheet",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/context?id=contexttextimagelinksafari"
    },
    "safari": {
      "!type": "object",
      "!doc": "Get parameters from Safari sharing",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/context?id=contexttextimagelinksafari"
    },
    "allItems": {
      "!type": "object",
      "!doc": "Get all items from share sheet",
      "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/vm/context?id=contexttextimagelinksafari"
    }
  },
  "btoa": {
    "!type": "fn(value: string) -> string",
    "!doc": "A shortcut that encodes a string to base64 string.",
    "!url": "https://developer.mozilla.org/en-US/docs/Web/API/WindowBase64/Base64_encoding_and_decoding"
  },
  "atob": {
    "!type": "fn(value: string) -> string",
    "!doc": "A shortcut that decodes a base64 encoded string.",
    "!url": "https://developer.mozilla.org/en-US/docs/Web/API/WindowBase64/Base64_encoding_and_decoding"
  },
  "alert": {
    "!type": "fn(args: ?)",
    "!doc": "A shortcut of the alert function in the ui module.",
    "!url": "https://cyanzhong.github.io/jsbox-nodejs/#/en/native-modules/ui"
  }
}